﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Resources;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Quick_Class_Model_Generator
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Multiselect = false;
            ofd.Filter = "Quick Class Model (*.qcm)|*.qcm";
            ofd.InitialDirectory = Application.ExecutablePath;
            if(ofd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    txtInput.Text = File.ReadAllText(ofd.FileName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "Quick Class Model (*.qcm)|*.qcm";
            sfd.InitialDirectory = Application.ExecutablePath;
            if(sfd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    File.WriteAllText(sfd.FileName, txtInput.Text);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.InitialDirectory = Application.ExecutablePath;
            sfd.FileName = ".cs";
            if (sfd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    File.WriteAllText(sfd.FileName, txtOutput.Text);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void button5_Click(object sender, EventArgs e)
        {
            try
            {
                Clipboard.SetText(txtOutput.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void cmbConstructorMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            chkStaticClass.Enabled = (cmbConstructorMode.SelectedIndex == 0);
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            Instances.Main = this;
            cmbConstructorMode.SelectedIndex = 1;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (txtInput.Lines.Length == 0 || String.IsNullOrWhiteSpace(txtClassname.Text)) { MessageBox.Show("Please enter some data!"); return; }

            List<String> tmpLines = new List<string>();
            tmpLines.AddRange(txtInput.Lines);

            List<String> lines = new List<string>();

            foreach(String line in tmpLines)
            {
                if(!String.IsNullOrEmpty(line) && line.Split().Length >= 3)
                {
                    lines.Add(line);
                }
            }

            List<Property> properties = new List<Property>();
            
            foreach(String line in lines)
            {
                properties.Add(Property.parsePropertyFromString(line));
            }

            txtOutput.Clear();

            StringBuilder output = new StringBuilder();
            output.AppendLine("public class " + txtClassname.Text);
            output.AppendLine("{");

            foreach(Property prop in properties)
            {
                output.Append(prop.getVariableDeclarationCode());
            }

            output.AppendLine("");
            output.AppendLine("");

            if(chkGenerateEvents.Checked)
            {
                foreach (Property prop in properties)
                {
                    output.Append(prop.getEventDeclarationCode());
                }

                output.AppendLine("");
                output.AppendLine("");

                foreach (Property prop in properties)
                {
                    output.Append(prop.getEventOnChangedArgsClassCode());
                    output.AppendLine("");
                }

                output.AppendLine("");
                output.AppendLine("");
            }

            if(cmbConstructorMode.SelectedIndex == 1)
            {
                    output.AppendLine("    public " + txtClassname.Text + "()");
                    output.AppendLine("    {");
                    output.AppendLine("");
                    output.AppendLine("    }");

                    output.AppendLine("");
                    output.AppendLine("");

                    switch(properties.Count)
                    {
                        case 1:
                            output.AppendLine("    public " + txtClassname.Text + "(" + properties[0].Datatype + " " + properties[0].Name.ToLower() + ")");
                            output.AppendLine("    {");
                            output.AppendLine("        this.m" + properties[0].Name + " = " + properties[0].Name.ToLower() + ";");
                            output.AppendLine("    }");
                            break;
                        default:
                            output.Append("    public " + txtClassname.Text + "(");
                            for (int i = 0; i < properties.Count - 1; i++)
                            {
                                output.Append(properties[i].Datatype + " " + properties[i].Name.ToLower() + ", ");
                            }
                            output.Append(properties[properties.Count - 1].Datatype + " " + properties[properties.Count - 1].Name.ToLower());
                            output.AppendLine(")");
                            output.AppendLine("    {");
                            foreach(Property prop in properties)
                            {
                                output.AppendLine("        this.m" + prop.Name + " = " + prop.Name.ToLower() + ";");
                            }
                            output.AppendLine("    }");
                            break;
                    }

                    output.AppendLine("");
                    output.AppendLine("");
            }

            foreach (Property prop in properties)
            {
                output.Append(prop.getPropertyCode());
                output.AppendLine("");
            }

            output.AppendLine("}");

            txtOutput.Text = output.ToString();
        }

        private void button6_Click(object sender, EventArgs e)
        {
            Process.GetCurrentProcess().Kill();
        }

        private void button7_Click(object sender, EventArgs e)
        {
            MessageBox.Show("This tool allows you to create class models fast and easy. " + Environment.NewLine + Environment.NewLine + "Here is some help (spaces are important!):" + Environment.NewLine + Environment.NewLine + "c <classname>" + Environment.NewLine + "<name> <datatype> <modifier> <access>" + Environment.NewLine + "<modifier>:" + Environment.NewLine + "public: pu" + Environment.NewLine + "private: pr" + Environment.NewLine + Environment.NewLine + "<access> (optional!):" + Environment.NewLine + "readonly: r" + Environment.NewLine + "writeonly: w" + Environment.NewLine + "all: Leave blank");
        }
    }
}
